# frozen_string_literal: true

require 'awesome_print'

module Sambot
  module Chef
    class Generator

      def self.from_templates(config, cloud, local_workflow, generated_files)
        generated_files.each { |template_name, opts| generate_from_template(template_name.to_s, opts, config) }
        generate_bootstrap_scripts(config, cloud, local_workflow)
      end

      private_class_method

      def self.bootstrap_centos(path)
        generate_bootstrap_from_template(path, 'sh')
      end

      def self.bootstrap_windows(path)
        generate_bootstrap_from_template(path, 'ps1')
      end

      def self.bootstrap(config, path)
        bootstrap_centos(path) if config.runs_on_centos?
        bootstrap_windows(path) if config.runs_on_windows?
      end

      def self.generate_bootstrap_from_template(path, suffix)
        Template.new("bootstrap_scripts/#{path}/bootstrap.#{suffix}.erb").process(eruby: true, dest: "bootstrap.#{suffix}")
      end

      def self.generate_bootstrap_scripts(config, cloud, local_workflow)
        cloud != 'local' ? bootstrap(config, cloud) : bootstrap(config, "local/#{local_workflow}")
      end

      def self.exists!(path)
        return if File.exist?(path) || Dir.exist?(path)
        raise ApplicationError, "The file or directory #{path} was not found in the current directory."
      end

      def self.generate_from_template(template_file, opts, config)
        UI.debug("Processing #{template_file} with opts #{opts.inspect} on platform #{config.available_platforms}")
        template = Template.new(template_file)
        if valid_platform?(opts, config)
          template.process(opts)
          UI.debug("#{opts[:dest]} has been added to the cookbook.")
        end
      end

      def self.valid_platform?(opts, config)
        targets = opts[:platform].map(&:to_s)
        result = targets & config.available_platforms
        result.size.positive?
      end

    end
  end
end
