# frozen_string_literal: true

require 'yaml'
require 'semantic'

module Sambot
  class Config

    CONFIGURATION_FILENAME = '.config.yml'

    def self.bump_version(path = nil)
      path ||= File.join(Dir.getwd, CONFIGURATION_FILENAME)
      contents = File.read(path)
      version = contents.match(/version: (.+)$/)[1]
      new_version = bump(version)
      contents = contents.gsub(version, new_version)
      File.write(path, contents)
      new_version
    end

    def initialize(opts)
      @opts = opts
    end

    def self.read(path = nil)
      path ||= File.join(Dir.getwd, CONFIGURATION_FILENAME)
      raise ApplicationError, "The configuration file was not found at #{path}." unless File.exist?(path)
      config = YAML.load_file(path)
      raise ApplicationError, 'Missing cookbook name in the .config.yml configuration file' unless config['name']
      raise ApplicationError, 'Missing platforms in the .config.yml configuration file' unless config['platform'] || config['platforms']
      raise ApplicationError, 'Missing version in the .config.yml configuration file' unless config['version']
      raise ApplicationError, 'Missing list of suites in the .config.yml configuration file' unless config['suites']
      raise ApplicationError, 'Missing description in the .config.yml configuration file' unless config['description']
      # Dealing with legacy tech debt of allowing multiple platforms rather than a single platform
      unless config['platforms']
        unless config['platform'].is_a?(Array)
          config['platform'] = [config['platform']]
        end
        config['platforms'] = config['platform']
      end
      Config.new(config)
    end

    def is_role_cookbook?
      name.start_with?('as-role-') || name.start_with?('as-app-role-')
    end

    def available_platforms
      platforms = @opts[:platforms] if @opts.key?(:platforms)
      platforms = @opts['platforms'] if @opts.key?('platforms')
      platforms
    end

    def forwarded_ports
      @opts['forwarded_ports'] || []
    end

    def gems
      @opts['gems'] || []
    end

    def dependencies
      items = @opts['dependencies']
      items ? items.map { |x| transform_hashes(x) } : []
    end

    def dependencies=(value)
      @opts['dependencies'] = value
    end

    def transform_hashes(obj)
      obj.is_a?(Hash) ? "#{obj.keys.first}', '#{obj.values.first}" : obj
    end

    def description; @opts['description']; end

    def identifier; @opts['identifier']; end

    def suites; @opts['suites']; end

    def version; @opts['version']; end

    def secrets; @opts.dig('local_testing', 'secrets') || []; end

    def name; @opts['name']; end

    def runs_on_centos?
      available_platforms.include?('centos')
    end

    def runs_on_windows?
      available_platforms.include?('windows')
    end

    private_class_method

    def self.bump(version)
      UI.debug("Old cookbook version: #{version}")
      version_info = Semantic::Version.new version
      new_version = "#{version_info.major}.#{version_info.minor}.#{version_info.patch + 1}"
      UI.debug("Old cookbook version: #{new_version}")
      new_version
    end

  end
end
